/*
 * Part of the PiyoPiyoJ project.
 * (PiyoPiyo Java)
 *
 *        DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *                    Version 2, December 2004 
 *
 * Modified by gamemanj <asdd2808@gmail.com> to remove expletives
 * Original copyright (C) 2004 Sam Hocevar <sam@hocevar.net> 
 *
 * Everyone is permitted to copy and distribute verbatim or modified 
 * copies of this license document, and changing it is allowed as long 
 * as the name is changed. 
 *
 *            DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION 
 *
 *  0. You just DO WHATEVER YOU WANT TO.
 */
package piyopiyoj.games.ui;

import gabien.*;
import gabien.music.PiyoPiyoFile;
import gabien.music.PiyoPiyoFrame;
import gabien.ui.IPointer;
import gabien.ui.OldMouseEmulator;
import gabien.ui.Size;
import gabien.ui.UIElement;

public class UINotesDisplay extends UIElement implements OldMouseEmulator.IOldMouseReceiver {
    public final PiyoPiyoFile file;
    public static final int fingerLen = 24;
    public static final int tileSize = 12;
    public int xOffset, track;
    public int selectedStart = 5, selectedEnd = 10;
    public OldMouseEmulator emulation = new OldMouseEmulator(this);

    public NoteClickCallback toolCallback = null;
    public boolean[] fingerInvert = new boolean[24];

    public interface NoteClickCallback {
        // Note that frameIndex doesn't include the track offset.
        void onClick(int frameIndex, int pos, int button);

        void onDrag(int frameIndex, int pos);

        void onRelease();

        void onFingerClick(int i, int button);
    }

    public UINotesDisplay(PiyoPiyoFile f) {
        super(640, 12 * (8 + 24));
        file = f;
    }

    @Override
    public void update(double deltaTime, boolean selected, IPeripherals peripherals) {
        Size elementBounds = getSize();

        // Autoexpand the file based upon what the user can see.
        int requiredSize = xOffset + (elementBounds.width / tileSize);
        int deficit = requiredSize - (file.musicFrames.length / 4);
        if (deficit > 0)
            file.resize(requiredSize + 64); // extra 64 for good measure, no point reallocating all the time
    }

    @Override
    public void render(IGrDriver igd) {
        igd.clearAll(0, 0, 0);
        IImage ui = GaBIEn.getImageCKEx("ui.png", false, true, 0, 0, 0);
        Size elementBounds = getSize();

        // Draw the notes on the left side.

        boolean[] sharp = {false, true, false, true, false, true, false,
                false, true, false, true, false, false, true, false, true,
                false, true, false, false, true, false, true, false};

        // Thanks to Jazz Jackalope for the idea and letter list.
        String[] key = {"B", "A#", "A", "G#", "G", "F#", "F",
                "E", "D#", "D", "C#", "C", "B", "A#", "A", "G#",
                "G", "F#", "F", "E", "D#", "D", "C#", "C"};
        for (int py = 0; py < (elementBounds.height / tileSize); py++)
            if (py < fingerInvert.length) {
                igd.blitImage(sharp[py] ? fingerLen : 0, tileSize, fingerLen, tileSize, 0, py * tileSize, ui);
                FontManager.drawString(igd, 1, 2 + (py * tileSize), key[py], true, fingerInvert[fingerInvert.length - (py + 1)], 8);
            }
        // Draw the note grid itself.
        for (int px = 0; px < ((elementBounds.width - fingerLen) / tileSize); px++) {
            if (px + xOffset < 0)
                continue;
            if (px + xOffset >= (file.musicFrames.length / 4))
                break;
            PiyoPiyoFrame[] frames = {
                    file.musicFrames[px + xOffset],
                    file.musicFrames[(px + xOffset) + (file.musicFrames.length / 4)],
                    file.musicFrames[(px + xOffset) + (2 * (file.musicFrames.length / 4))],
                    file.musicFrames[(px + xOffset) + (3 * (file.musicFrames.length / 4))]
            };
            for (int py = 0; py < (elementBounds.height / tileSize); py++) {
                if (py >= 24) {
                    if (py >= 32)
                        continue;
                    igd.blitImage(tileSize * ((py == 24) ? 6 : 5), 0, tileSize, tileSize, fingerLen + (px * tileSize), py * tileSize, ui);
                    if (frames[track].panValue == (py - 24))
                        igd.blitImage(tileSize, 0, tileSize, tileSize, fingerLen + (px * tileSize), py * tileSize, ui);
                    continue;
                }
                boolean cellOn = frames[track].hitNotes[23 - py];
                igd.blitImage(cellOn ? tileSize : 0, 0, tileSize, tileSize, fingerLen + (px * tileSize), py * tileSize, ui);
                if (file.loopEnd == (px + xOffset))
                    igd.blitImage(tileSize * 2, 0, tileSize, tileSize, fingerLen + (px * tileSize), py * tileSize, ui);
                if (file.loopStart == (px + xOffset))
                    igd.blitImage(tileSize * 3, 0, tileSize, tileSize, fingerLen + (px * tileSize), py * tileSize, ui);
                if ((px + xOffset) >= selectedStart)
                    if ((px + xOffset) <= selectedEnd)
                        igd.blitImage(tileSize * 4, 0, tileSize, tileSize, fingerLen + (px * tileSize), py * tileSize, ui);
            }
        }
    }

    @Override
    public void handleClick(int x, int y, int button) {
        if (x < fingerLen) {
            if ((y / tileSize) >= 24)
                return;
            toolCallback.onFingerClick(23 - (y / tileSize), button);
            return;
        }
        int ox = ((x - fingerLen) / tileSize) + xOffset;
        if (ox < 0)
            return;
        if (ox >= (file.musicFrames.length / 4))
            return;
        if ((y / tileSize) >= 24) {
            if ((y / tileSize) < 32)
                adjustPan(ox, y / tileSize);
            return;
        }
        if (toolCallback != null)
            toolCallback.onClick(ox, 23 - (y / tileSize), button);
    }

    @Override
    public void handleDrag(int x, int y) {
        if (x < fingerLen)
            return;
        int ox = ((x - fingerLen) / tileSize) + xOffset;
        if (ox < 0)
            return;
        if (ox >= (file.musicFrames.length / 4))
            return;
        if ((y / tileSize) >= 24) {
            if ((y / tileSize) < 32)
                adjustPan(ox, y / tileSize);
            return;
        }
        if (toolCallback != null)
            toolCallback.onDrag(ox, 23 - (y / tileSize));
    }

    @Override
    public void handleRelease(int x, int y) {
        if (toolCallback != null)
            toolCallback.onRelease();
    }

    @Override
    public void handlePointerBegin(IPointer state) {
        emulation.handlePointerBegin(state);
    }

    @Override
    public void handlePointerUpdate(IPointer state) {
        emulation.handlePointerUpdate(state);
    }

    @Override
    public void handlePointerEnd(IPointer state) {
        emulation.handlePointerEnd(state);
    }

    private void adjustPan(int ox, int i) {
        file.musicFrames[ox + (track * (file.musicFrames.length / 4))].panValue = i - 24;
    }

}
