/*
 * Part of the GaBIEn project.
 * (Graphics and Basic Input Engine)
 *
 *        DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *                    Version 2, December 2004 
 *
 * Modified by gamemanj <asdd2808@gmail.com> to remove expletives
 * Original copyright (C) 2004 Sam Hocevar <sam@hocevar.net> 
 *
 * Everyone is permitted to copy and distribute verbatim or modified 
 * copies of this license document, and changing it is allowed as long 
 * as the name is changed. 
 *
 *            DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION 
 *
 *  0. You just DO WHATEVER YOU WANT TO.
 */
package gabien.music;

import gabien.SimpleMixer;

import java.io.IOException;
import java.io.InputStream;
import java.util.LinkedList;

public class PiyoPiyoPlayer {

    // Notes & Samples
    public LinkedList<PiyoPiyoPlayingNote> playingNotes = new LinkedList<PiyoPiyoPlayingNote>();
    public LinkedList<PiyoPiyoPlayingSample> playingSamples = new LinkedList<PiyoPiyoPlayingSample>();

    public double withinBeatPosition = 0;
    public final PiyoPiyoFile piyoFile;
    public SimpleMixer sound;
    public int currentFrame = 0;
    // If false, then loop start/end is ignored.
    public boolean handleLoop = true;

    public int[] panValues = new int[4];

    public double pitchScale = 1.0d;

    public PiyoPiyoPlayer(InputStream fis, SimpleMixer sound)
            throws IOException {
        this.sound = sound;
        piyoFile = new PiyoPiyoFile();
        piyoFile.loadFile(fis);
    }

    public PiyoPiyoPlayer(PiyoPiyoFile file, SimpleMixer sound) {
        this.sound = sound;
        piyoFile = file;
        panValues[0] = 3;
        panValues[1] = 3;
        panValues[2] = 3;
        panValues[3] = 3;
    }

    // Update beats. Can be run in reverse.
    public void updateBeats(double deltaTime) {
        withinBeatPosition += deltaTime;
        double musicWaitTime = Math.max(piyoFile.beatTime(), 0.001d);
        while (withinBeatPosition < 0) {
            withinBeatPosition += musicWaitTime;
            for (int i = 0; i < 4; i++)
                handleBeat(piyoFile.musicFrames[currentFrame + ((piyoFile.musicFrames.length / 4) * i)], i);
            currentFrame--;
            if (currentFrame < 0)
                setCurrentFrame(piyoFile.loopEnd);
        }
        while (withinBeatPosition > musicWaitTime) {
            withinBeatPosition -= musicWaitTime;
            for (int i = 0; i < 4; i++)
                handleBeat(piyoFile.musicFrames[currentFrame + ((piyoFile.musicFrames.length / 4) * i)], i);
            currentFrame++;
            if (((currentFrame * 4) >= piyoFile.musicFrames.length) || (handleLoop && (currentFrame >= piyoFile.loopEnd)))
                setCurrentFrame(piyoFile.loopStart);
        }
    }

    // Update envelopes. Cannot be run in reverse.
    public void updateEnvelopes(double deltaTime) {
        for (PiyoPiyoPlayingNote note : new LinkedList<PiyoPiyoPlayingNote>(
                playingNotes)) {
            if (note.updateEnvelope(deltaTime)) {
                sound.deleteChannel(note.channel);
                playingNotes.remove(note);
            }
        }
        for (PiyoPiyoPlayingSample note : new LinkedList<PiyoPiyoPlayingSample>(
                playingSamples)) {
            if (note.doUpdate(deltaTime)) {
                sound.deleteChannel(note.channel);
                playingSamples.remove(note);
            }
        }
    }

    public void setCurrentFrame(int value) {
        currentFrame = value;
        if (currentFrame < 0)
            currentFrame = 0;
        if ((currentFrame * 4) >= piyoFile.musicFrames.length)
            currentFrame = (piyoFile.musicFrames.length / 4) - 1;
    }

    private void handleBeat(PiyoPiyoFrame piyoPiyoFrame, int track) {
        if (piyoPiyoFrame.panValue != 0)
            panValues[track] = piyoPiyoFrame.panValue - 1;
        for (int i = 0; i < 24; i++)
            if (piyoPiyoFrame.hitNotes[i])
                playNote(track, i, panValues[track]);
    }

    public void playNote(int track, int i, int panValue) {
        double globalVolume = 0.5d;
        switch (track) {
            case 0:
            case 1:
            case 2:
                playingNotes.add(new PiyoPiyoPlayingNote(sound.createChannel(), piyoFile.waveTracks[track], i, panValue, globalVolume, pitchScale));
                break;
            case 3:
                playingSamples.add(new PiyoPiyoPlayingSample(sound.createChannel(), i, piyoFile.drumVolume, panValue, globalVolume, pitchScale));
                break;
            default:
                break;
        }
    }

    public void stopAll() {
        for (PiyoPiyoPlayingNote note : playingNotes)
            sound.deleteChannel(note.channel);
        for (PiyoPiyoPlayingSample note : playingSamples)
            sound.deleteChannel(note.channel);
        playingNotes.clear();
        playingSamples.clear();
        panValues[0] = 3;
        panValues[1] = 3;
        panValues[2] = 3;
        panValues[3] = 3;
    }
}
