/*
 * Part of the PiyoPiyoJ project.
 * (PiyoPiyo Java)
 *
 *        DO WHATEVER YOU WANT TO PUBLIC LICENSE
 *                    Version 2, December 2004
 *
 * Modified by gamemanj <asdd2808@gmail.com> to remove expletives
 * Original copyright (C) 2004 Sam Hocevar <sam@hocevar.net>
 *
 * Everyone is permitted to copy and distribute verbatim or modified
 * copies of this license document, and changing it is allowed as long
 * as the name is changed.
 *
 *            DO WHATEVER YOU WANT TO PUBLIC LICENSE
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
 *
 *  0. You just DO WHATEVER YOU WANT TO.
 */

package piyopiyoj.games;

import gabien.SimpleMixer;
import gabien.music.PiyoPiyoFile;
import gabien.music.PiyoPiyoPlayer;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class WavOut {
    public static void perform(OutputStream fos, PiyoPiyoFile coreFile, int startFrame, int ignoreFrame, int frameCount) throws IOException {
        SimpleMixer sm = new SimpleMixer(true, true);
        PiyoPiyoPlayer iPlayer = new PiyoPiyoPlayer(coreFile, sm);
        double time = 1d / 44100d;
        // Halve pitch to account for stepup to 44.1k
        iPlayer.pitchScale = 0.5d;
        iPlayer.setCurrentFrame(startFrame);
        // Regarding frameCount: Some complex loops could fool a more flexible approach.
        int ignoreSampleCount = (int) (ignoreFrame * 44100 * iPlayer.piyoFile.beatTime());
        int sampleCount = (int) (frameCount * 44100 * iPlayer.piyoFile.beatTime());
        int sampleTotal = sampleCount * 4;

        // Now we know exactly what we're going to generate, let's build a header!
        int headSize = 52;
        ByteBuffer bb = ByteBuffer.allocate(headSize);
        bb.order(ByteOrder.LITTLE_ENDIAN);
        // Header start...
        bb.put((byte) 'R');
        bb.put((byte) 'I');
        bb.put((byte) 'F');
        bb.put((byte) 'F');
        // What's the rest of the file going to be?
        bb.putInt(headSize + sampleTotal - 8);
        // Filetype
        bb.put((byte) 'W');
        bb.put((byte) 'A');
        bb.put((byte) 'V');
        bb.put((byte) 'E');
        bb.put((byte) '/');
        bb.put((byte) 'P');
        bb.put((byte) 'P');
        bb.put((byte) 'J');
        bb.putInt(0);
        bb.put((byte) 'f');
        bb.put((byte) 'm');
        bb.put((byte) 't');
        bb.put((byte) ' ');
        bb.putInt(0x10);
        // fmt...
        bb.putInt(0x00020001);
        bb.putInt(0x0000AC44);
        bb.putInt(0x0002B110);
        bb.putInt(0x00100004);
        bb.put((byte) 'd');
        bb.put((byte) 'a');
        bb.put((byte) 't');
        bb.put((byte) 'a');
        bb.putInt(sampleTotal);
        fos.write(bb.array());
        for (int i = 0; i < ignoreSampleCount; i += 32) {
            iPlayer.updateEnvelopes(time * 32);
            iPlayer.updateBeats(time * 32);
            // Stereo data.
            sm.pullData(Math.min(ignoreSampleCount - i, 32));
        }
        // The reason this is done in sections of 2 is to speed up the process.
        for (int i = 0; i < sampleCount; i++) {
            iPlayer.updateEnvelopes(time * 2);
            iPlayer.updateBeats(time * 2);
            // Stereo data.
            short[] samples = sm.pullData(2);
            for (int j = 0; j < samples.length; j += 2) {
                int s = (samples[j] & 0xFFFF) | ((samples[j + 1] & 0xFFFF) << 16);
                bb.position(0);
                bb.putInt(s);
                fos.write(bb.array(), 0, 4);
            }
        }
    }
}
